/**
 * Module dependencies
 */

var http = require('http')
  , fs = require('fs')
  , path = require('path');

var config = require(path.join(process.env.PWD, 'config', 'application'))
  , controllers = (function () {
      var cons = fs.readdirSync(path.join(process.env.PWD, config.paths.controllers))
        , obj = {};
      cons.forEach(function (file) {
        var name = path.basename(file, '.js');
        obj[name] = require(path.join(process.env.PWD, config.paths.controllers, file));
      });
      return obj;
    }())
  , routes = require(path.join(process.env.PWD, 'config', 'routes'))
  , _index = -1
  , _req
  , _res
  , _out;

function next(err) {
  _index += 1;
  if (err) next(err);
  var layer = application.middleware[_index];
  if (!layer) {
    if (_out) _out(err);
    if (err) {
      console.log(err);
    }
    return;
  } else {
    layer.handle(_req, _res, next);
  }
}

var application = function application() {

  this.config = config;

  this.controllers = controllers;

  this.middleware = application.middleware = (function () {
    var layers = [];
    config.middleware.forEach(function (layer) {
      var name = Object.keys(layer)[0]
        , mw = require('./middleware/' + name);
      layers.push({ handle: mw(layer[name].options) });
    });
    return layers;
  }());

  this.routes = routes;

  if (application.caller !== application.getInstance) {
    throw new Error("This object cannot be instantiated.");
  }
};

application.instance = null;

application.getInstance = function () {
  if (this.instance === null) {
    this.instance = new application();
  }
  return this.instance;
};

application.prototype.handle = function (req, res, out) {
  _index = -1;
  _req = req;
  _res = res;
  _out = out;
  next();
};

application.prototype.listen = function () {
  var server = http.createServer(this.handle);
  return server.listen.apply(server, arguments);
};

module.exports = application.getInstance();
